
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.geom.Ellipse2D;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JPanel;

public class PainelPontos extends JPanel {

	public ArrayList<Ponto> pontosDaCurva,pontosDeBezierC1,pontosDeBezierC2,curvaDeBezierC1,curvaDeBezierC2;
	public boolean desenharLinhas;

	public PainelPontos(boolean desenharLinhas) {

		if(pontosDaCurva == null){
			pontosDaCurva = new ArrayList<Ponto>();
		}
		if(pontosDeBezierC1 == null){
			pontosDeBezierC1 = new ArrayList<Ponto>();
		}
		if(pontosDeBezierC2 == null){
			pontosDeBezierC2 = new ArrayList<Ponto>();
		}
		if(curvaDeBezierC1 == null){
			curvaDeBezierC1 = new ArrayList<Ponto>();
		}

		this.setLayout(null);
		this.setBorder(BorderFactory.createEtchedBorder());
		this.setPreferredSize(new Dimension (600, 800));
		this.setVisible(true);

		this.desenharLinhas = desenharLinhas;

		this.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent e) {
				int x = e.getX();
				int y = e.getY();

				Ponto p = new Ponto(x,y,"Entrada");
				
				calcularPontosControleBezierC1(p);
				calcularPontosControleBezierC2(p);
				
				// Limpa a tela
				((JPanel)e.getComponent()).removeAll();	

				pontosDaCurva.add(p);

				// Adiciona os pontos de curva na tela
				for(Ponto pc : pontosDaCurva){
					add(pc,null);
				}

				if(pontosDaCurva.size() > 2){
					curvaDeBezierC1.clear();
					
					for(int i = 2; i < pontosDeBezierC1.size(); i += 2){
						for (double t = 0; t <= 1; t += 0.01)
							curvaDeBezierC1.add( calcularBezierC1(i-2,t) );
					}					
				}
				
				if (pontosDeBezierC1.size() > 3) {
					for (int i = 0; i+3 < pontosDeBezierC1.size(); i++){
						for (double t = 0; t < 1; t = t + 0.05) {
							calcularBezierC2(i,t);
						}
					}
				}
				
				// Adiciona os pontos de bezier na tela
				for(Ponto pb : pontosDeBezierC1){
					add(pb,null);
				}

				//  Adiciona a curva de bezier na tela
				for(Ponto cb : curvaDeBezierC1){
					add(cb,null);
				}
				
				((JPanel)e.getComponent().getParent()).repaint();	
			}
		});
	}

	private void calcularPontosControleBezierC1(Ponto pTemp) {
		Ponto p5 = new Ponto(pTemp.x,pTemp.y,"Bezier");

		if(pontosDaCurva.size() < 3){
			pontosDeBezierC1.add(p5);
		}
		else{
			Ponto p2 = getAndRemoveLast();
			Ponto p1 = pontosDeBezierC1.get( pontosDeBezierC1.size() - 1);			
			Ponto p12 = p1.interpolacaoLinear(p2);
						
			pontosDeBezierC1.add(p12);
			pontosDeBezierC1.add(p2);
			pontosDeBezierC1.add(p5);
		}
	}

	private void calcularPontosControleBezierC2(Ponto pTemp) {
		Ponto p5 = new Ponto(pTemp.x,pTemp.y,"Bezier");

		if(pontosDaCurva.size() < 3){
			pontosDeBezierC1.add(p5);
		}
		else if(pontosDaCurva.size() < 5){

			Ponto p2 = getAndRemoveLast();			
			Ponto p1 = pontosDeBezierC1.get( pontosDeBezierC1.size() - 1);			
			Ponto p12 = p1.interpolacaoLinear(p2);

			if(pontosDaCurva.size() == 4) getAndRemoveLast(); 

			pontosDeBezierC1.add(p12);
			pontosDeBezierC1.add(p2);
			pontosDeBezierC1.add(p5);
		}
		else{
			Ponto p4 = getAndRemoveLast();
			Ponto p3 = getAndRemoveLast();
			getAndRemoveLast();

			Ponto p2 = pontosDaCurva.get(pontosDaCurva.size()-3);

			Ponto p23_1 = p2.getPrimeiroPontoInterpolacao(p3);
			Ponto p23_2 = p2.getSegundoPontoInterpolacao(p3);

			pontosDeBezierC1.add(p23_1);
			pontosDeBezierC1.add(p23_2);

			Ponto p34 = p3.interpolacaoLinear(p4);
			pontosDeBezierC1.add(p34);

			pontosDeBezierC1.add(p4);
			pontosDeBezierC1.add(p5);
		}

	}
	 
	public void paintComponent(Graphics g) {
		Graphics2D ga = (Graphics2D)g;

		Color c = ga.getBackground();

		if(desenharLinhas)
			ga.setColor(Color.BLUE);
		else
			ga.setColor(c);

		int index = 0;

		for(Ponto p : pontosDaCurva){
			if(index > 0){
				Ponto anterior = pontosDaCurva.get(index-1);
				ga.drawLine((int)p.x, (int)p.y, (int)anterior.x, (int)anterior.y);
			}
			index++;
		}
	}

	public Ponto getAndRemoveLast(){
		Ponto last = pontosDeBezierC1.get(pontosDeBezierC1.size()-1);

		// Remove do vetor
		pontosDeBezierC1.remove(pontosDeBezierC1.size()-1);
		return last;
	}

	public Ponto calcularBezierC1(int index,double t) {

		Ponto a = pontosDeBezierC1.get(index);
		Ponto b = pontosDeBezierC1.get(index+1);
		Ponto c = pontosDeBezierC1.get(index+2);
		
		Ponto bezier = c.bezierC1(a, b, t);
		
		curvaDeBezierC1.add(bezier);
		
		return bezier;
	}
	
	public void calcularBezierC2(int i, double t) {
		Ponto a = pontosDeBezierC1.get(i);
		Ponto b = pontosDeBezierC1.get(i+1);
		Ponto c = pontosDeBezierC1.get(i+2);
		Ponto d = pontosDeBezierC1.get(i+3);
		
		Ponto daCurva = d.bezierC2(a, b, c, t);
		curvaDeBezierC1.add(daCurva);
		
	}
}
